package com.chelvc.framework.database.mapper;

import java.util.List;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.metadata.TableFieldInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.annotation.CreateUpdate;
import lombok.NonNull;

/**
 * 插入或修改方法实现
 *
 * @author Woody
 * @date 2024/2/27
 */
public class InsertUpdateMethod extends AbstractInsertMethod {
    public InsertUpdateMethod() {
        super("insertUpdate");
    }

    @Override
    protected String initializeMethodScript(@NonNull TableInfo table) {
        List<TableFieldInfo> fields = table.getFieldList();
        String columns = this.field2column(table, fields);
        String properties = this.field2property(table, fields);
        String updates = fields.stream().map(field -> {
            CreateUpdate annotation = field.getField().getAnnotation(CreateUpdate.class);
            if (annotation != null && !annotation.enabled()) {
                return null;
            }
            StringBuilder buffer = new StringBuilder().append(field.getColumn()).append(" = ");
            CreateUpdate.Mode mode = annotation == null ? CreateUpdate.Mode.REPLACE : annotation.mode();
            if (mode == CreateUpdate.Mode.REPLACE) {
                buffer.append("VALUES(").append(field.getColumn()).append(")");
            } else if (mode == CreateUpdate.Mode.INCREMENT) {
                buffer.append(field.getColumn()).append(" + 1");
            } else if (mode == CreateUpdate.Mode.DECREMENT) {
                buffer.append(field.getColumn()).append(" - 1");
            } else {
                throw new IllegalArgumentException("Not support update mode: " + mode);
            }
            return buffer.toString();
        }).filter(StringUtils::notEmpty).collect(Collectors.joining(StringPool.COMMA));
        return String.format(
                "<script>\nINSERT IGNORE INTO %s (%s) VALUES (%s)\n ON DUPLICATE KEY UPDATE %s\n</script>",
                table.getTableName(), columns, properties, updates
        );
    }
}
